/****************************************************************************
 **                                                                        **
 **    File:           argparse.c                                          **
 **    Author:         Roger Preston (modifications only)                  **
 **    Created/Copied: Wed Dec 14 18:11:21 EST 1988                        **
 **    Contents :      Argument parser                                     **
 **                                                                        **
 **    Copyright (c) 1988 BBN Communications Corp.                         **
 **    All rights reserved.                                                **
 **                                                                        **
 **    Argument parsing for schematic plotter program.                     **
 **    Pinched from a Steve Geyer program and hacked in a major            **
 **    way by guy fedorkow at bbn, Oct/86.                                 **
 **    (Taken by rmp and hacked even more for GP1000 stress test)          **
 **                                                                        **
 ****************************************************************************/
#include <stdio.h>
#include "tmp4.h"
#include "tmp3.h"
#include "tmp2.h"

extern struct bugfix switches[];
extern int in_file_number;
extern char *g_input_file[NFILES];

char *progname;  /* global to this file only */
long atoxl();

/*
 * The arg parser uses two types of tables... one is a table
 * that specifies each recognized flag, along with some indication
 * of what to do with it.
 * One of the things to do is look for an argument for a flag that
 * matches one in a list, e.g. "-forms legal".  These lists are
 * kept in tables of the second type.
 *
 * Names that are not attached to arguments are assumed to be
 * input file names.
 *
 * The list of input file names collected on the command line is
 * kept in a fixed size array.  The last item in the list is followed
 * by a null entry.
 */


/*****************************************************************************
 *                                                                           *
 * argparse -- parse the command line arguments passed to the program;       *
 *             ac and av are the argc, argv parms passed by main.            *
 *             nfiles is the limit on number of input files that can be      *
 *             accepted.  May be set to 0 or 1 or anything less than NFILES  *
 *             by the application  program                                   *
 *                                                                           *
 *****************************************************************************/
argparse(ac, av, nfiles)
register int ac;
register char **av;
int nfiles;
{
  register struct switches *pntr;
  auto     struct switches *found;

  for (in_file_number = NFILES; in_file_number > 0; )
    g_input_file[--in_file_number] = NULL;
  
  if (nfiles > NFILES -1)
    ecmderr(0, "!!recompile with larger NFILES!!\n");
  
  progname = *av++; --ac;
  
  for (found = NULL; ac != 0; ++av, --ac, found = NULL)
  {
    for (pntr = (struct switches *) switches; pntr->sw_name != NULL; pntr++)
    {
      if (sveq_nocase(*av, pntr->sw_name))
	if (found == NULL)
	  found = pntr;
	else
	  ecmderr (0, "ambiguous switch name.\n'%s' matches %s and %s.\n",
		   *av, found->sw_name, pntr->sw_name);
    }
      
    if (found == NULL)                      /* Default action */
    {
      if (*av[0] == '-')
	ecmderr(0, "unknown switch: %s\n", *av);
		  
      if (nfiles == 0)
	ecmderr(0, "no input files allowed\n");

/* What's going on here??? */
	  
      if (in_file_number == nfiles)
	ecmderr(0, "no more than %d input file%s allowed\n",
		nfiles, plural(nfiles));
	  
      g_input_file[in_file_number++] = *av;
	  
      continue;
    }
      
    if (found->sw_parms > ac - 1)
      ecmderr (0, "'%s' requires %d parameter%s.\n", 
	       found->sw_name, found->sw_parms, plural(found->sw_parms), *av);

    switch (found->sw_type)
    {
      case SW_LIST:
        if (find_in_list(*++av, found->sw_listp, found->sw_addr.sw_int)
                 == FALSE)
        ecmderr(0, "unknown value for switch '%s'\n", *(av - 1));
        ac--;
        break;
      
      case SW_FLAG:
        *found->sw_addr.sw_flag = TRUE;
        break;
      
      case SW_FUNC:
        (*found->sw_addr.sw_func)(av);
        av += found->sw_parms;
        ac -= found->sw_parms;
        break;
      
      case SW_CHAR:
        *found->sw_addr.sw_char = atoi(*++av);
        --ac;
        break;
      
      case SW_INT:
        *found->sw_addr.sw_int = atoi(*++av);
        --ac;
        break;
      
      case SW_HEX:
        *found->sw_addr.sw_int = atox(*++av);
        --ac;
        break;
      
      case SW_LONG_HEX:
        *found->sw_addr.sw_long = atoxl(*++av);
        --ac;
        break;
      
      case SW_STRING:
        *found->sw_addr.sw_string = *++av;
        --ac;
        break;
    }
  }
}



/*****************************************************************************
 * find_in_list - compare an argument to a list of strings.  If there's      *
 * a match, write the associated value at the integer pointer passed         *
 * along with the string.  Return TRUE if there's a unique match, FALSE      *
 * if the match fails.                                                       *
 *****************************************************************************/
find_in_list(str, listp, valp)
char *str;
struct sw_list *listp;
int *valp;
{
  struct sw_list *p, *found;
  
  found = NULL;
  for (p = listp; p->arg_name != NULL; p++) 
    if (sveq_nocase(str, p->arg_name))
    {
      if (found == NULL)
	found = p;
      else
	return(FALSE);
    }
  
  if (found == NULL)
    return(FALSE);
  
  *valp = found->arg_val;
  return(TRUE);
}

			
/*****************************************************************************
 * sveq_nocase - like seq_nocase, but if str1 is an initial substring of     *
 *    str2 it is considered equal rather than less than str2.  This is use-  *
 *    ful for partial sequence recognition.  Returns TRUE if they are equal, *
 *    FALSE otherwise.  NULL is treated as an empty string.  If str1 is      *
 *    empty it is always less than str2, rather than equal.                  *
 *****************************************************************************/
sveq_nocase (str1, str2)
char   *str1;
char   *str2;
{
  return (! svcmp_nocase (str1, str2));
}



/*****************************************************************************
 * svcmp_nocase - compares two strings in a case independent fashion, but    *
 * returns equal (0) if the first string is an initial substring of the      *
 * second string.  This is useful for partial command recognition.  Returns  *
 * <0 if str1 < str2,  ==0 if str1==str2,  and >0 if str1 > str2.  Null is   *
 * is treated as an empty string.  If str1 is empty it is always less than   *
 * str2, rather than equal.                                                  *
 *****************************************************************************/

#define upper(c)        (('a' <= (c) && (c) <= 'z') ? (c) - 'a' + 'A' : (c))

svcmp_nocase (str1, str2)
register char  *str1;
register char  *str2;
{
  register char   c1, c2;

  /* if the first string is NULL (or empty) return 'less-than'       */
  /* so empty strings never match anything.                          */
  
  if ((str1 == NULL) || (*str1 == NULL))
    return (-1);
  
  if ((str2 == NULL) || (*str2 == NULL))
    return (1);
  
  do
  {
    c1 = *str1++;
    c2 = *str2++;
    c1 = upper (c1);
    c2 = upper (c2);
    if (c1 == NULL)
      return (0);
  }
  while (c1 == c2);
  
  return (c1 - c2);
}


/*****************************************************************************
 *                                                                           *
 * print a 'help' message that lists all the command args and options        *
 *                                                                           *
 *****************************************************************************/
print_help() 
{
  struct bugfix *swp;
  struct sw_list *lp;
  
  fprintf(stderr, "%s:", progname);
  for (swp = &switches[0]; swp->sw_name != NULL; swp++) 
  {
    switch (swp->sw_type) 
    {
    case SW_INT:
    case SW_CHAR:
      fprintf(stderr, "   %s number\n", swp->sw_name);
      break;
      
    case SW_HEX:
    case SW_LONG_HEX:
      fprintf(stderr, "   %s hex number\n", swp->sw_name);
      break;
      
    case SW_STRING:
      fprintf(stderr, "   %s name\n", swp->sw_name);
      break;
      
    case SW_FLAG:
    case SW_FUNC:
      fprintf(stderr, "   %s", swp->sw_name);
      if (swp->sw_parms)
	fprintf(stderr, " [%d args]", swp->sw_parms);
      fprintf(stderr, "\n");
      break;
      
    case SW_LIST:
      fprintf(stderr, "   %s [", swp->sw_name);
      for (lp = swp->sw_listp; lp->arg_name != NULL; lp++)
	fprintf(stderr, "%s ", lp->arg_name);
      fprintf(stderr, "]\n");
      break;
    }
  }
  exit(0);
}

/******************************************************************************
 * find_name_from_val -- take the value associated with an arg string and     *
 *   convert it to the name for that argument by scanning the list of pos-    *
 *   sible args                                                               *
 *****************************************************************************/
char * find_name_from_val(listp, val)
struct sw_list *listp;
int val;
{
  struct sw_list *p;
  
  for (p = listp; p->arg_name != NULL; p++) 
  {
    if (p->arg_val == val)
      return(p->arg_name);
  }
  return("");  /* couldn't find it */
}


/******************************************************************************
 * a not-very-smooth version of cmderr added by guy fedorkow                  *
 * to make the port to Sun work                                               *
 *****************************************************************************/
ecmderr(code, str, a, b, c, d, e, f, g)
int code;
char *str;
int a, b, c, d, e, f, g;
{
  fprintf(stderr, "%s: ", progname);
  fprintf(stderr, str, a, b, c, d, e, f, g);
  exit(1);
}


/*****************************************************************************
 * hex number converter pinched from chrysalis library                       *
 ****************************************************************************/
atox(s)
char    *s;
{
  int	     sum;
  int	     digit;
  char     c;
  
  sum = 0;
  while (c = *s++)
  {
    if (c >= '0' && c <= '9')
      digit = c - '0';
    else if (c >= 'a' && c <= 'f')
      digit = c - 'a' + 10;
    else if (c >= 'A' && c <= 'F')
      digit = c - 'A' + 10;
    else
      break;
    
    sum <<= 4;
    sum += digit;
  }
  
  return (sum);
}

long atoxl(s)
char    *s;
{
  long     sum;
  int	     digit;
  char     c;
  
  sum = 0L;
  while (c = *s++)
  {
    if (c >= '0' && c <= '9')
      digit = c - '0';
    else if (c >= 'a' && c <= 'f')
      digit = c - 'a' + 10;
    else if (c >= 'A' && c <= 'F')
      digit = c - 'A' + 10;
    else
      break;
    
    sum <<= 4;
    sum += digit;
  }
  
  return (sum);
}

