/****************************************************************************
 **                                                                        **
 **    File:           strings.c                                           **
 **    Author:         Roger Preston                                       **
 **    Created/Copied: Wed Dec 14 17:11:51 EST 1988                        **
 **    Contents :      String handling routines                            **
 **                                                                        **
 **    Copyright (c) 1988 BBN Communications Corp.                         **
 **    All rights reserved.                                                **
 **                                                                        **
 ** This module pinched from the bisynchronous pad command processor       **
 ** Written by g fedorkow, 1984                                            **
 ** modified by fedorkow, june/86                                          **
 **                                                                        **
 ****************************************************************************/
#include "tmp1.h"
#include "tmp4.h"

/****************************************************************************
 * lcase() --                                                               *
 ****************************************************************************/
char lcase(c)
    register char c;
    {
    if (c >= 'A' && c <= 'Z')
	return(c + ('a' - 'A'));
    if (c == '_')
	return('-');
    return(c);
}

/****************************************************************************
 * is_alpha() --                                                            *
 ****************************************************************************/
is_alpha(c)
    register char c;
    {
    if ((c >= 'a' && c <= 'z')  ||
	(c >= 'A' && c <= 'Z'))
	return(TRUE);
    return(FALSE);
}

/****************************************************************************
 * is_numeric()                                                             *
 ****************************************************************************/
is_numeric(c)
    register char c;
    {
    if (c >= '0' && c <= '9')  
	return(TRUE);
    return(FALSE);
}

/****************************************************************************
 * smatch() -- test to see if the first argument is a substring of the      *
 * second.  If so, return SUBSTRING.  If the string is an exact copy of the *
 * template, return EXACT_MATCH.  Otherwise, return FALSE.                  *
 ****************************************************************************/
smatch(test, template)
    register char *test;         /*a potential command or argument */
    register char *template;     /*a template to compare against */
    {
    while(*test) {
	if (lcase(*test++) != lcase(*template++))
	    return(FALSE);
    }
    if (*template == '\0')
	return(EXACT_MATCH);
    return(SUBSTRING);
}


/****************************************************************************
 * search_list() -- Test to see if the first argument is a unique subtsring *
 * of one of the strings given by the list named in the second argument.    *
 ****************************************************************************/
search_list(test, list, size)
    register char *test;
    register char **list;
    register int  size;
    {
    register short i;
    register short matches, list_id;
    register char  **p;         /* p is a pointer to a pointer */
    size /= sizeof(char *);     /* since p points to pointers  */
                                /* arithmetic on p will assume */
                                /* an implicit size of 4       */

    i=0;
    matches = 0;
    for (p = list; *p != 0; p += size) {
	switch (smatch(test, *p)) {
	    case SUBSTRING:
    	        matches ++;
                list_id = i;
		break;

	    case EXACT_MATCH:
		return(i);
	}
        i++;
    }
    if (matches == 1)
	return(list_id);
    return(UNDEFINED);
}


/****************************************************************************
 * scopy() --                                                               *
 ****************************************************************************/
char *scopy(src, dest)
    register char *src, *dest;
    {
    while (*src)
	*dest++ = *src++;

    *dest = '\0';
    return(dest);
}


/****************************************************************************
 * nscopy() -- copy a string of up to count characters.  Call this routine  *
 * with count == -1 to copy any (reasonable) length string.                 *
 ****************************************************************************/
char *nscopy(src, dest, count)
    register char *src, *dest;
    short count;
    {
    while (*src  &&  count-- != 0)
	*dest++ = *src++;
    
    *dest = '\0';
    return(dest);
}


/****************************************************************************
 * seq() --                                                                 *
 ****************************************************************************/
seq(s1, s2)
    register char *s1, *s2;
    {
    if (s1 == s2)
	return(TRUE);
    if (s1 == 0  ||  s2 == 0)
 	 return(FALSE);
    while(*s1 == *s2++)
	if (*s1++ == '\0')
	    return(TRUE);
    return(FALSE);
}





/****************************************************************************
 * atoiv() -- converts the string passed in the first argument into an      *
 * integer, which is placed in the location pointed to by the second.       *
 * It returns 0 if the conversion was successful.  -1 is returned           *
 * if the string contained non-numeric data.                                *
 * The third argument gives the base to use for conversion.  10 and         *
 * 16 are the regular values.                                               *
 * We assume that blanks have been stripped elsewhere.                      *
 * Note: For 8086-class machines, the result goes into a LONG,              *
 * not an INT.                                                              *
 ****************************************************************************/
atoiv(str, ip, base)
    char *str;
    long *ip;
    int base;
    {
    long result;
    register int sign, i;

    *ip = 0L;
    result = 0L;
#ifdef SIGNED_CONVERSION
    if (*str == '-') {
	str++;
	sign = -1;
    }
    else
#endif
	sign = 1;

    if (*str == '\0')
	return(ATOI_NULL);

    while (*str) {
	if ((i = convert(*str, base)) < 0)
	    return(ATOI_BAD);
	result = result * base + i;
	str++;
    }
    if (sign < 0)
	result = -result;
    *ip = result;
    return(ATOI_GOOD);
}


/****************************************************************************
 * convert() -- convert a single character into an integer. Return -1 if    *
 * the character is not legal in that base.                                 *
 ****************************************************************************/
convert(c, base)
    char c;
    int base;
    {
    int i;

    i = -1;
    if (c >= '0'  && c <= '9')
	i = c - '0';

    if (c >= 'A' && c <= 'F')
	i = c - 'A' + 10;

    if (c >= 'a' && c <= 'f')
	i = c - 'a' + 10;

    if (i >= base)
	return(-1);
    return(i);
}







